/*
 *
 *  * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 *  * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 *
 */

package ru.ulstu.controllers;

import io.swagger.annotations.ApiOperation;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import ru.ulstu.configuration.ApiConfiguration;
import ru.ulstu.models.ForecastParams;
import ru.ulstu.models.ModelingResult;
import ru.ulstu.models.TimeSeries;
import ru.ulstu.models.exceptions.ModelingException;
import ru.ulstu.services.MethodParamBruteForce;
import ru.ulstu.services.TimeSeriesService;

@RestController
@RequestMapping(ApiConfiguration.API_1_0)
public class TimeSeriesController {

    private final TimeSeriesService timeSeriesService;
    private final MethodParamBruteForce methodParamBruteForce;

    public TimeSeriesController(TimeSeriesService timeSeriesService,
                                MethodParamBruteForce methodParamBruteForce) {
        this.timeSeriesService = timeSeriesService;
        this.methodParamBruteForce = methodParamBruteForce;
    }

    @PostMapping("getForecast")
    @ApiOperation("Получить прогноз временного ряда")
    public ResponseEntity<TimeSeries> getForecastTimeSeries(@RequestBody ForecastParams forecastParams) throws ModelingException {
        return new ResponseEntity<>(timeSeriesService.getForecast(forecastParams.getOriginalTimeSeries(),
                forecastParams.getCountForecast()), HttpStatus.OK);
    }

    @PostMapping("getSmoothed")
    @ApiOperation("Получить сглаженный временной ряд")
    public ResponseEntity<ModelingResult> getSmoothedTimeSeries(@RequestBody TimeSeries timeSeries) throws ModelingException {
        return new ResponseEntity<>(methodParamBruteForce.getSmoothedTimeSeries(timeSeries), HttpStatus.OK);
    }
}
