/*
 *
 *  * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 *  * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 *
 */

package ru.ulstu.services;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.ulstu.models.TimeSeries;
import ru.ulstu.models.exceptions.ModelingException;
import ru.ulstu.tsMethods.TimeSeriesMethod;
import ru.ulstu.tsMethods.exponential.AddTrendAddSeason;
import ru.ulstu.tsMethods.exponential.NoTrendNoSeason;
import ru.ulstu.tsMethods.exponential.parameter.Alpha;
import ru.ulstu.tsMethods.exponential.parameter.ExponentialMethodParamValue;


@Service
public class TimeSeriesService {
    private static final Logger LOGGER = LoggerFactory.getLogger(TimeSeriesService.class);

    public TimeSeries getForecast(TimeSeries timeSeries, int countPoints) throws ModelingException {
        TimeSeriesMethod method;
        method = new NoTrendNoSeason().createFor(timeSeries).setAlpha(new ExponentialMethodParamValue<>(new Alpha(), 0.8));
        method = new AddTrendAddSeason().createFor(timeSeries);
        return method.getForecast(countPoints);
    }

    public TimeSeries smoothTimeSeries(TimeSeries timeSeries) throws ModelingException {
        //NoTrendNoSeason nn = new NoTrendNoSeason(timeSeries, ExponentialMethodParams.of(ExponentialParamName.ALPHA, 0.8));
        TimeSeriesMethod method;
        method = new NoTrendNoSeason().createFor(timeSeries);
        method = new AddTrendAddSeason().createFor(timeSeries);
        return method.getModel();
    }
}
