/*
 *
 *  * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 *  * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 *
 */

package ru.ulstu.tsMethods.exponential;

import org.springframework.stereotype.Component;
import ru.ulstu.models.TimeSeries;
import ru.ulstu.tsMethods.MethodParameter;
import ru.ulstu.tsMethods.TimeSeriesMethod;
import ru.ulstu.tsMethods.TimeSeriesMethodParamValue;
import ru.ulstu.tsMethods.exponential.parameter.Alpha;
import ru.ulstu.tsMethods.exponential.parameter.Beta;
import ru.ulstu.tsMethods.exponential.parameter.ExponentialMethodParamValue;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Component
public class AddTrendNoSeason extends TimeSeriesMethod {
    private ExponentialMethodParamValue<Alpha> alpha = new ExponentialMethodParamValue<>(Alpha.getInstance(), 0.5);
    private ExponentialMethodParamValue<Beta> beta = new ExponentialMethodParamValue<>(Beta.getInstance(), 0.5);
    private final List<Double> sComponent = new ArrayList<>();
    private final List<Double> tComponent = new ArrayList<>();

    public AddTrendNoSeason createFor(TimeSeries timeSeries) {
        this.originalTimeSeries = timeSeries;
        return this;
    }

    public AddTrendNoSeason setAlpha(ExponentialMethodParamValue<Alpha> alpha) {
        this.alpha = alpha;
        return this;
    }

    public AddTrendNoSeason setBeta(ExponentialMethodParamValue<Beta> beta) {
        this.beta = beta;
        return this;
    }

    @Override
    protected TimeSeries getModelOfValidTimeSeries() {
        sComponent.clear();
        tComponent.clear();
        sComponent.add(originalTimeSeries.getFirstValue().getValue());
        tComponent.add(originalTimeSeries.getValues().get(1).getValue() - originalTimeSeries.getValues().get(0).getValue());
        TimeSeries model = new TimeSeries("Model of " + originalTimeSeries.getName());
        model.addValue(originalTimeSeries.getFirstValue());
        //выполняется проход модели по сглаживанию
        for (int t = 1; t < originalTimeSeries.getValues().size(); t++) {
            sComponent.add(alpha.getDoubleValue() * originalTimeSeries.getNumericValue(t)
                    + (1 - alpha.getDoubleValue())
                    * (sComponent.get(t - 1) - tComponent.get(t - 1)));

            tComponent.add(beta.getDoubleValue()
                    * (sComponent.get(t) - sComponent.get(t - 1))
                    + (1 - beta.getDoubleValue()) * tComponent.get(t - 1));
            model.addValue(originalTimeSeries.getValues().get(t), sComponent.get(sComponent.size() - 1));
        }
        return model;
    }

    @Override
    protected TimeSeries makeForecast(TimeSeries forecast) {
        for (int t = 1; t < forecast.getLength(); t++) {
            forecast.getValues().get(t).setValue(sComponent.get(sComponent.size() - 1) + tComponent.get(tComponent.size() - 1) * t);
        }
        return forecast;
    }

    @Override
    public List<MethodParameter> getAvailableParameters() {
        return Arrays.asList(alpha.getParam(), beta.getParam());
    }

    @Override
    public TimeSeriesMethod setAvailableParameters(List<TimeSeriesMethodParamValue> parameters) {
        for (TimeSeriesMethodParamValue parameter : parameters) {
            if (parameter.getParameter() instanceof Alpha) {
                alpha.setValue(parameter.getValue());
            }
            if (parameter.getParameter() instanceof Beta) {
                beta.setValue(parameter.getValue());
            }
        }
        return this;
    }
}
