/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 */

package ru.ulstu.tsMethods.exponential;

import org.springframework.stereotype.Component;
import ru.ulstu.models.TimeSeries;
import ru.ulstu.tsMethods.MethodParameter;
import ru.ulstu.tsMethods.TimeSeriesMethod;
import ru.ulstu.tsMethods.TimeSeriesMethodParamValue;
import ru.ulstu.tsMethods.exponential.parameter.Alpha;
import ru.ulstu.tsMethods.exponential.parameter.ExponentialMethodParamValue;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Component
public class NoTrendNoSeason extends TimeSeriesMethod {
    private ExponentialMethodParamValue<Alpha> alpha = new ExponentialMethodParamValue<>(Alpha.getInstance(), 0.5);
    private final List<Double> sComponent = new ArrayList<>();

    public NoTrendNoSeason createFor(TimeSeries timeSeries) {
        this.originalTimeSeries = timeSeries;
        return this;
    }

    public NoTrendNoSeason setAlpha(ExponentialMethodParamValue<Alpha> alpha) {
        this.alpha = alpha;
        return this;
    }

    @Override
    protected TimeSeries getModelOfValidTimeSeries() {
        sComponent.clear();
        sComponent.add(originalTimeSeries.getFirstValue().getValue());
        TimeSeries model = new TimeSeries("Model of " + originalTimeSeries.getName());
        model.addValue(originalTimeSeries.getFirstValue());
        //выполняется проход модели по сглаживанию
        for (int t = 1; t < originalTimeSeries.getValues().size(); t++) {
            sComponent.add(sComponent.get(t - 1)
                    + alpha.getDoubleValue()
                    * (originalTimeSeries.getNumericValue(t) - sComponent.get(t - 1)));
            model.addValue(originalTimeSeries.getValues().get(t), sComponent.get(sComponent.size() - 1));
        }
        return model;
    }

    @Override
    protected TimeSeries makeForecast(TimeSeries forecast) {
        for (int t = 1; t < forecast.getLength(); t++) {
            forecast.getValues().get(t).setValue(forecast.getValues().get(t - 1).getValue());
        }
        return forecast;
    }

    @Override
    public List<MethodParameter> getAvailableParameters() {
        return Collections.singletonList(alpha.getParam());
    }

    @Override
    public TimeSeriesMethod setAvailableParameters(List<TimeSeriesMethodParamValue> parameters) {
        for (TimeSeriesMethodParamValue parameter : parameters) {
            if (parameter.getParameter() instanceof Alpha) {
                alpha.setValue(parameter.getValue());
            }
        }
        return this;
    }
}
